@@ -21,70 +21,6 @@ try:
 except ImportError:
     Spectral11 = None
 
-class Status_Monitor(object):
-    """ Display the tasks running and waiting on each worker
-
-    Parameters
-    ----------
-    addr: tuple, optional
-        (ip, port) of scheduler.  Defaults to scheduler of recent Executor
-    interval: Number, optional
-        Interval between updates.  Defaults to 1s
-    """
-    def __init__(self, addr=None, interval=1000.00, loop=None):
-        if addr is None:
-            scheduler = default_executor().scheduler
-            if isinstance(scheduler, rpc):
-                addr = (scheduler.ip, 9786)
-            elif isinstance(scheduler, Scheduler):
-                addr = ('127.0.0.1', scheduler.services['http'].port)
-        self.addr = addr
-        self.interval = interval
-
-        self.display_notebook = False
-
-        if is_kernel() and not curstate().notebook:
-            output_notebook()
-            assert curstate().notebook
-
-        self.task_source, self.task_table = task_table_plot()
-        self.worker_source, self.worker_table = worker_table_plot()
-
-        self.output = vplot(self.worker_table, self.task_table)
-
-        self.client = AsyncHTTPClient()
-
-        self.loop = loop or IOLoop.current()
-        self.loop.add_callback(self.update)
-        self._pc = PeriodicCallback(self.update, self.interval, io_loop=self.loop)
-        self._pc.start()
-
-    def _ipython_display_(self, **kwargs):
-        show(self.output)
-        self.display_notebook = True
-
-    @gen.coroutine
-    def update(self):
-        """ Query the Scheduler, update the figure
-
-        This opens a connection to the scheduler, sends it a function to run
-        periodically, streams the results back and uses those results to update
-        the bokeh figure
-        """
-        with log_errors():
-            tasks, workers = yield [
-                    self.client.fetch('http://%s:%d/tasks.json' % self.addr),
-                    self.client.fetch('http://%s:%d/workers.json' % self.addr)]
-
-            tasks = json.loads(tasks.body.decode())
-            workers = json.loads(workers.body.decode())
-
-            task_table_update(self.task_source, tasks)
-            worker_table_update(self.worker_source, workers)
-
-            if self.display_notebook:
-                push_notebook()
-
 
 def task_table_plot(row_headers=False, width=600, height=400):
     names = ['waiting', 'ready', 'failed', 'processing', 'in-memory', 'total']
@@ -248,7 +184,7 @@ def task_stream_append(lists, msg, workers, palette=task_stream_palette):
 
 def progress_plot(**kwargs):
     from ..diagnostics.progress_stream import progress_quads
-    data = progress_quads({'all': {}, 'in_memory': {},
+    data = progress_quads({'all': {}, 'memory': {},
                            'erred': {}, 'released': {}})
 
     x_range = Range1d(-0.5, 1.5)
@@ -260,7 +196,7 @@ def progress_plot(**kwargs):
     fig.quad(source=source, top='top', bottom='bottom',
              left=0, right='released_right', color=Spectral9[0], alpha=0.4)
     fig.quad(source=source, top='top', bottom='bottom',
-             left='released_right', right='in_memory_right',
+             left='released_right', right='memory_right',
              color=Spectral9[0], alpha=0.8)
     fig.quad(source=source, top='top', bottom='bottom',
              left='erred_left', right=1,
@@ -288,7 +224,7 @@ def progress_plot(**kwargs):
     </div>
     <div>
         <span style="font-size: 14px; font-weight: bold;">In Memory:</span>&nbsp;
-        <span style="font-size: 10px; font-family: Monaco, monospace;">@in_memory</span>
+        <span style="font-size: 10px; font-family: Monaco, monospace;">@memory</span>
     </div>
     <div>
         <span style="font-size: 14px; font-weight: bold;">Erred:</span>&nbsp;
