@@ -5,17 +5,21 @@ from multiprocessing import Process
 import socket
 
 from tornado import gen, ioloop
+from tornado.iostream import StreamClosedError
 import pytest
 
 from distributed.core import (read, write, pingpong, Server, rpc, connect,
-        coerce_to_rpc)
+        coerce_to_rpc, send_recv, coerce_to_address)
 from distributed.utils_test import slow, loop, gen_test
 
 def test_server(loop):
     @gen.coroutine
     def f():
         server = Server({'ping': pingpong})
+        with pytest.raises(OSError):
+            server.port
         server.listen(8887)
+        assert server.port == 8887
 
         stream = yield connect('127.0.0.1', 8887)
 
@@ -87,12 +91,12 @@ def test_rpc_with_many_connections(loop):
     loop.run_sync(f)
 
 
+def echo(stream, x):
+    return x
+
 @slow
 def test_large_packets(loop):
     """ tornado has a 100MB cap by default """
-    def echo(stream, x):
-        return x
-
     @gen.coroutine
     def f():
         server = Server({'echo': echo})
@@ -181,3 +185,37 @@ def test_errors():
         yield r.div(x=1, y=0)
 
     r.close_streams()
+
+
+@gen_test()
+def test_connect_raises():
+    with pytest.raises((gen.TimeoutError, StreamClosedError)):
+        yield connect('127.0.0.1', 58259, timeout=0.01)
+
+
+@gen_test()
+def test_send_recv_args():
+    server = Server({'echo': echo})
+    server.listen(0)
+
+    result = yield send_recv(arg=('127.0.0.1', server.port), op='echo', x=b'1')
+    assert result == b'1'
+    result = yield send_recv(addr=('127.0.0.1:%d' % server.port).encode(),
+                             op='echo', x=b'1')
+    assert result == b'1'
+    result = yield send_recv(ip=b'127.0.0.1', port=server.port, op='echo',
+                            x=b'1')
+    assert result == b'1'
+    result = yield send_recv(ip=b'127.0.0.1', port=server.port, op='echo',
+                             x=b'1', reply=False)
+    assert result == None
+
+    server.stop()
+
+
+def test_coerce_to_address():
+    for arg in [b'127.0.0.1:8786',
+                '127.0.0.1:8786',
+                ('127.0.0.1', 8786),
+                ('127.0.0.1', '8786')]:
+        assert coerce_to_address(arg) == '127.0.0.1:8786'
