@@ -8,6 +8,7 @@ import sys
 from time import time, sleep
 
 import dask
+from dask import delayed
 from dask.core import get_deps
 from toolz import merge, concat, valmap, first
 from tornado.queues import Queue
@@ -23,6 +24,7 @@ from distributed.core import connect, read, write, rpc, dumps
 from distributed.client import WrappedKey
 from distributed.scheduler import (validate_state, decide_worker,
         Scheduler)
+from distributed.executor import _wait
 from distributed.worker import dumps_function, dumps_task
 from distributed.utils_test import (inc, ignoring, dec, gen_cluster, gen_test,
         loop)
@@ -34,6 +36,12 @@ alice = 'alice:1234'
 bob = 'bob:1234'
 
 
+@gen_cluster()
+def test_administration(s, a, b):
+    assert isinstance(s.address, str)
+    assert s.address_tuple[0] in s.address
+
+
 @gen_cluster()
 def test_ready_add_worker(s, a, b):
     s.add_client(client='client')
@@ -53,9 +61,11 @@ def test_update_state(s):
                    dependencies={'y': 'x', 'x': set()},
                    client='client')
 
-    s.mark_task_finished('x', alice, nbytes=10, type=dumps(int),
-            compute_start=10, compute_stop=11)
-    s.ensure_occupied(alice)
+    s.ensure_occupied()
+    r = s.transition('x', 'memory', nbytes=10, type=dumps(int),
+            compute_start=10, compute_stop=11, worker=alice)
+    s.transitions(r)
+    s.ensure_occupied()
 
     assert set(s.processing[alice]) == {'y'}
     assert set(s.rprocessing['y']) == {alice}
@@ -90,9 +100,11 @@ def test_update_state_with_processing(s):
                    dependencies={'y': {'x'}, 'x': set(), 'z': {'y'}},
                    client='client')
 
-    s.mark_task_finished('x', alice, nbytes=10, type=dumps(int),
-            compute_start=10, compute_stop=11)
-    s.ensure_occupied(alice)
+    s.ensure_occupied()
+    r = s.transition('x', 'memory', nbytes=10, type=dumps(int),
+            compute_start=10, compute_stop=11, worker=alice)
+    s.transitions(r)
+    s.ensure_occupied()
 
     assert s.waiting == {'z': {'y'}}
     assert s.waiting_data == {'x': {'y'}, 'y': {'z'}, 'z': set()}
@@ -118,36 +130,21 @@ def test_update_state_with_processing(s):
     assert s.wants_what == {'client': {'b', 'c', 'z'}}
 
 
-@gen_cluster(ncores=[])
-def test_update_state_respects_data_in_memory(s):
-    s.add_worker(address=alice, ncores=1, coerce_address=False)
-    s.update_graph(tasks={'x': 1, 'y': (inc, 'x')},
-                   keys=['y'],
-                   dependencies={'y': {'x'}, 'x': set()},
-                   client='client')
+@gen_cluster(executor=True, ncores=[('127.0.0.1', 1)])
+def test_update_state_respects_data_in_memory(e, s, a):
+    x = delayed(inc)(1)
+    y = delayed(inc)(x)
+    f = e.persist(y)
+    yield _wait([f])
 
-    s.mark_task_finished('x', alice, nbytes=10, type=dumps(int),
-                         compute_start=10, compute_stop=11)
-    s.ensure_occupied(alice)
-    s.mark_task_finished('y', alice, nbytes=10, type=dumps(int),
-                         compute_start=11, compute_stop=12)
-    s.ensure_occupied(alice)
+    assert s.released == {x.key}
+    assert s.who_has == {y.key: {a.address}}
 
-    assert s.released == {'x'}
-    assert s.who_has == {'y': {alice}}
-
-    s.update_graph(tasks={'x': 1, 'y': (inc, 'x'), 'z': (add, 'y', 'x')},
-                   keys=['z'],
-                   dependencies={'y': {'x'}, 'z': {'y', 'x'}},
-                   client='client')
-
-    assert s.released == set()
-    assert s.waiting == {'z': {'x'}}
-    assert set(s.processing[alice]) == {'x'}  # x was released need to recompute
-    assert set(s.rprocessing['x']) == {alice}  # x was released need to recompute
-    assert s.waiting_data == {'x': {'z'}, 'y': {'z'}, 'z': set()}
-    assert s.who_wants == {'y': {'client'}, 'z': {'client'}}
-    assert s.wants_what == {'client': {'y', 'z'}}
+    z = delayed(add)(x, y)
+    f2 = e.persist(z)
+    while not f2.key in s.who_has:
+        assert y.key in s.who_has
+        yield gen.sleep(0.0001)
 
 
 @gen_cluster(ncores=[])
@@ -158,15 +155,19 @@ def test_update_state_supports_recomputing_released_results(s):
                    dependencies={'y': {'x'}, 'x': set(), 'z': {'y'}},
                    client='client')
 
-    s.mark_task_finished('x', alice, nbytes=10, type=dumps(int),
-                         compute_start=10, compute_stop=11)
-    s.ensure_occupied(alice)
-    s.mark_task_finished('y', alice, nbytes=10, type=dumps(int),
-                         compute_start=10, compute_stop=11)
-    s.ensure_occupied(alice)
-    s.mark_task_finished('z', alice, nbytes=10, type=dumps(int),
-                         compute_start=10, compute_stop=11)
-    s.ensure_occupied(alice)
+    s.ensure_occupied()
+    r = s.transition('x', 'memory', nbytes=10, type=dumps(int),
+            compute_start=10, compute_stop=11, worker=alice)
+    s.transitions(r)
+    s.ensure_occupied()
+    r = s.transition('y', 'memory', nbytes=10, type=dumps(int),
+            compute_start=10, compute_stop=11, worker=alice)
+    s.transitions(r)
+    s.ensure_occupied()
+    r = s.transition('z', 'memory', nbytes=10, type=dumps(int),
+            compute_start=10, compute_stop=11, worker=alice)
+    s.transitions(r)
+    s.ensure_occupied()
 
     assert not s.waiting
     assert not s.ready
@@ -176,7 +177,7 @@ def test_update_state_supports_recomputing_released_results(s):
 
     s.update_graph(tasks={'x': 1, 'y': (inc, 'x')},
                    keys=['y'],
-                   dependencies={'y': {'x'}},
+                   dependencies={'y': {'x'}, 'x': set()},
                    client='client')
 
     assert s.waiting == {'y': {'x'}}
@@ -281,6 +282,7 @@ def test_validate_state():
     in_play = {'x', 'y'}
     who_wants = {'y': {'client'}}
     wants_what = {'client': {'y'}}
+    erred = {}
 
     validate_state(**locals())
 
@@ -376,7 +378,7 @@ def test_scheduler(s, a, b):
 
     # Test missing data
     yield write(stream, {'op': 'missing-data', 'keys': ['z']})
-    s.ensure_idle_ready()
+    s.ensure_occupied()
 
     while True:
         msg = yield read(stream)
@@ -504,7 +506,7 @@ def test_remove_worker_from_scheduler(s, a, b):
     assert a.address not in s.ncores
     assert len(s.ready) + len(s.processing[b.address]) == \
             len(dsk)  # b owns everything
-    s.validate()
+    s.validate_state()
 
 
 @gen_cluster()
@@ -521,7 +523,7 @@ def test_add_worker(s, a, b):
     s.add_worker(address=w.address, keys=list(w.data),
                  ncores=w.ncores, services=s.services, coerce_address=False)
 
-    s.validate()
+    s.validate_state()
 
     assert w.ip in s.host_info
     assert s.host_info[w.ip]['ports'] == set(map(str, [a.port, b.port, w.port]))
@@ -577,7 +579,7 @@ def test_feed_setup_teardown(s, a, b):
 
 @gen_test(timeout=None)
 def test_scheduler_as_center():
-    s = Scheduler()
+    s = Scheduler(validate=True)
     done = s.start(0)
     a = Worker('127.0.0.1', s.port, ip='127.0.0.1', ncores=1)
     a.data.update({'x': 1, 'y': 2})
@@ -593,7 +595,7 @@ def test_scheduler_as_center():
                    keys=['a'],
                    dependencies={'a': []})
     start = time()
-    while not s.who_has['a']:
+    while not 'a' in s.who_has:
         assert time() - start < 5
         yield gen.sleep(0.01)
     assert 'a' in a.data or 'a' in b.data or 'a' in c.data
@@ -607,17 +609,21 @@ def test_scheduler_as_center():
     yield s.close()
 
 
-@gen_cluster()
-def test_delete_data(s, a, b):
-    yield s.scatter(data=valmap(dumps, {'x': 1, 'y': 2, 'z': 3}),
-                    client='client')
+@gen_cluster(executor=True)
+def test_delete_data(e, s, a, b):
+    d = yield e._scatter({'x': 1, 'y': 2, 'z': 3})
+
     assert set(s.who_has) == {'x', 'y', 'z'}
     assert set(a.data) | set(b.data) == {'x', 'y', 'z'}
     assert merge(a.data, b.data) == {'x': 1, 'y': 2, 'z': 3}
 
-    s.delete_data(keys=['x', 'y'])
-    yield s.clear_data_from_workers()
-    assert set(a.data) | set(b.data) == {'z'}
+    del d['x']
+    del d['y']
+
+    start = time()
+    while set(a.data) | set(b.data) != {'z'}:
+        yield gen.sleep(0.01)
+        assert time() < start + 5
 
 
 @gen_cluster()
@@ -629,15 +635,17 @@ def test_rpc(s, a, b):
     assert aa is not bb
 
 
-@gen_cluster()
-def test_delete_callback(s, a, b):
-    a.data['x'] = 1
-    s.who_has['x'].add(a.address)
-    s.has_what[a.address].add('x')
+@gen_cluster(executor=True)
+def test_delete_callback(e, s, a, b):
+    d = yield e._scatter({'x': 1}, workers=a.address)
 
-    s.delete_data(keys=['x'])
-    assert not s.who_has['x']
-    assert not s.has_what['y']
+    assert s.who_has['x'] == {a.address}
+    assert s.has_what[a.address] == {'x'}
+
+    s.client_releases_keys(client=e.id, keys=['x'])
+
+    assert 'x' not in s.who_has
+    assert 'x' not in s.has_what[a.address]
     assert a.data['x'] == 1  # still in memory
     assert s.deleted_keys == {a.address: {'x'}}
     yield s.clear_data_from_workers()
@@ -810,7 +818,7 @@ def test_ready_add_worker(s, a, b):
 
 @gen_test()
 def test_worker_name():
-    s = Scheduler()
+    s = Scheduler(validate=True)
     s.start(0)
     w = Worker(s.ip, s.port, name='alice')
     yield w._start()
@@ -827,7 +835,7 @@ def test_worker_name():
 
 @gen_test()
 def test_coerce_address():
-    s = Scheduler()
+    s = Scheduler(validate=True)
     s.start(0)
     a = Worker(s.ip, s.port, name='alice')
     b = Worker(s.ip, s.port, name=123)
@@ -933,5 +941,22 @@ def test_add_worker_is_idempotent(s):
 
 
 def test_io_loop(loop):
-    s = Scheduler(loop=loop)
+    s = Scheduler(loop=loop, validate=True)
     assert s.io_loop is loop
+
+
+@gen_cluster(executor=True)
+def test_transition_story(e, s, a, b):
+    x = delayed(inc)(1)
+    y = delayed(inc)(x)
+    f = e.persist(y)
+    yield _wait([f])
+
+    assert s.transition_log
+
+    story = s.transition_story(x.key)
+    assert all(line in s.transition_log for line in story)
+    assert len(story) < len(s.transition_log)
+    assert all(x.key == line[0] or x.key in line[-1] for line in story)
+
+    assert len(s.transition_story(x.key, y.key)) > len(story)
