@@ -25,6 +25,26 @@ import pytest
 
 logger = logging.getLogger(__name__)
 
+@pytest.fixture(scope='session')
+def valid_python_script(tmpdir_factory):
+    local_file = tmpdir_factory.mktemp('data').join('file.py')
+    local_file.write("print('hello world!')")
+    return local_file
+
+@pytest.fixture(scope='session')
+def executor_contract_script(tmpdir_factory):
+    local_file = tmpdir_factory.mktemp('data').join('distributed_script.py')
+    lines = ("from distributed import Executor", "e = Executor('127.0.0.1:8989')",
+     'print(e)')
+    local_file.write('\n'.join(lines))
+    return local_file
+
+@pytest.fixture(scope='session')
+def invalid_python_script(tmpdir_factory):
+    local_file = tmpdir_factory.mktemp('data').join('file.py')
+    local_file.write("a+1")
+    return local_file
+
 
 @pytest.yield_fixture
 def current_loop():
@@ -140,7 +160,7 @@ def run_scheduler(q, scheduler_port=0, center_port=None, **kwargs):
     logging.getLogger("tornado").setLevel(logging.CRITICAL)
 
     center = ('127.0.0.1', center_port) if center_port else None
-    scheduler = Scheduler(center=center, loop=loop, **kwargs)
+    scheduler = Scheduler(center=center, loop=loop, validate=True, **kwargs)
     done = scheduler.start(scheduler_port)
 
     q.put(scheduler.port)
@@ -369,7 +389,7 @@ def _test_scheduler(f, loop=None, b_ip='127.0.0.1'):
     from .executor import _global_executor
     @gen.coroutine
     def g():
-        s = Scheduler(ip='127.0.0.1')
+        s = Scheduler(ip='127.0.0.1', validate=True)
         done = s.start(0)
         a = Worker('127.0.0.1', s.port, ncores=2, ip='127.0.0.1')
         yield a._start()
@@ -427,7 +447,7 @@ from .executor import Executor
 
 @gen.coroutine
 def start_cluster(ncores, loop, Worker=Worker):
-    s = Scheduler(ip='127.0.0.1', loop=loop)
+    s = Scheduler(ip='127.0.0.1', loop=loop, validate=True)
     done = s.start(0)
     workers = [Worker(s.ip, s.port, ncores=v, ip=k, name=i, loop=loop)
                 for i, (k, v) in enumerate(ncores)]
